#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <malloc.h>
#include <errno.h>

extern int debug_flag;
extern char *mcamrc_dir;


int readline(FILE *file, char *contents)
{
int c, i;

for(i = 0; i < 512; i++)
	{
	c = getc(file);
	if(c == EOF)
		{
		fclose(file);
		contents[i] = 0;/* EOF marker */
		return(EOF);
		}
	if(c == '\n')
		{
		contents[i] = 0;/* EOF marker */
		return(1);/* end of line */
		}
	contents[i] = c;
	}
return(0);
}


struct setup
	{
	char *name;
	char *def;
	struct setup *nxtentr;
	struct setup *prventr;
	};
static struct setup *setuptab[2];


struct setup *lookup_setup(char *name, int sequence)
{
struct setup *pa;
int j;

j = 0;
/*pa points to next entry*/
for(pa = setuptab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0)
		{
		if(sequence == j) return(pa);/* found sequence entry */
		j++;
		}
	}
return(0);/*not found*/
}


struct setup *install_setup_at_end_of_list(char *name, char *def)
{
struct setup *plast, *pnew;
struct setup *lookup_setup();
extern char *strsave();

pnew = lookup_setup(name, 0);
if(pnew)
	{
	/* free previous def */
	free(pnew -> def);
	pnew -> def = strsave(def);
	if(! pnew -> def) return 0;
	return(pnew);
	}

/* create new structure */
pnew = (struct setup *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);

/* get previous structure */
plast = setuptab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previous structure pointers */
if( !setuptab[0] ) setuptab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
setuptab[1] = pnew;

pnew -> def = strsave(def);
if(! pnew -> def) return 0;

return(pnew);/* pointer to new structure */
}/* end function install_setup_at_end_of_list */


struct setup *install_setup(char *name, char *def)
{
struct setup *pa, *lookup_setup();
char *strsave();
int i;

pa = lookup_setup(name, 0);
if(pa)
	{
	/* free previous def */
	free(pa -> def);
	pa -> def = strsave(def);
	if(! pa -> def) return 0;

	return pa;	
	}

pa = (struct setup *) malloc( sizeof(*pa) );
if(!pa) return(0);

pa -> name = strsave(name);
if(! pa -> name) return(0);

pa -> def = strsave(def);
if(!pa -> def) return(0);

pa -> nxtentr = setuptab[0];
setuptab[0] = pa;

return(pa);
}


int set_setup(char *name, char *def)
{
struct setup *install_setup();
extern char *home_dir;

if(debug_flag) fprintf(stderr, "set_setup(): installing %s %s\n", name, def);

if(! install_setup_at_end_of_list(name, def) ) return(0);

if(! save_settings(home_dir) ) return 0;

return(1);
}


int get_setup(char *name, char *def, int sequence)
{
struct setup *pa, *lookup_setup();
struct setup *install_setup(), *install_setup_at_end_of_list();

if(debug_flag)
	{
	fprintf(stderr,\
	"get_setup(): arg name=%s sequence=%d\n", name, sequence);
	}

pa = lookup_setup(name, sequence);
if(! pa) return(0);

strcpy(def, pa -> def);
return(1);
}


int load_setup(char *home_dir)
{
int a;
char pathfilename[1024];
int setup_loaded_flag;
FILE *setupfile;
char temp[1024];
char *cp;
char arg0[512], arg1[512];

/* combine with home directory */
sprintf(pathfilename, "%s/.mcamrc", mcamrc_dir);
setupfile = fopen(pathfilename, "r");
if(! setupfile)/* no setup file present */
	{
	fprintf(stderr, "\nCannot open setup file %s\n", pathfilename);
	strerror(errno);
	return 0;
	}

setup_loaded_flag = 0;
/* read data from uirc.setup setupfile into database */
while(1)
	{
	/* read a line from the setup file */
	a = readline(setupfile, temp);
	if(a == EOF)break;

	/* lines starting with '#' are scipped */
	if(temp[0] == '#') continue; 

	/* scip comments to end of line */
	cp = strstr(temp, "//");
	/* force a string termination on "//" */
	if(cp) *cp = 0;
			
	/* get the start of the second field */
	a = sscanf(temp, "%s %s", arg0, arg1);

	/* scip empty lines */
	if (a == -1) continue;

	/* test if any second field was present,
	/* need at least 2 arguments for a valid entry.
	*/
/*	if( (a < 2) || (! set_setup(arg0, strstr(temp, arg1) ) ) )
fails on compression on, because on is also in end compression
*/
	if( (a < 2) || (! set_setup(arg0, arg1) ) )

		{
		fprintf(stderr,\
"\nCannot install data from setupfile in database, line reads %s\n", temp);
		if(setupfile) fclose(setupfile);
		return 0;
		}
	setup_loaded_flag = 1;
	}/* end while read all lines from setupfile */

return setup_loaded_flag;
}/* end function load setup */


int save_settings(char *home_dir)
{
int a;
char pathfilename_temp[1024];
char pathfilename_bak[1024];
char pathfilename[1024];
int setup_loaded_flag;
FILE *setupfile;
char temp[1024];
char *cp;
char arg0[512], arg1[512];
struct setup *pa;

if(debug_flag)
	{
	fprintf(stderr, "save_settings(): arg home_dir=%s\n", home_dir);
	}

if(! home_dir) return 0;

/* combine with home directory */
sprintf(pathfilename_temp, "%s/.mcamrc.tmp", mcamrc_dir);
setupfile = fopen(pathfilename_temp, "w");
if(! setupfile)
	{
	fprintf(stderr, "\nCannot open setup file %s\n", pathfilename_temp);
	strerror(errno);
	return 0;
	}

for(pa = setuptab[0]; pa != 0; pa = pa -> nxtentr)
	{
	fprintf(setupfile, "%s %s\n", pa -> name, pa -> def);
	}

fclose(setupfile);

sprintf(pathfilename, "%s/.mcamrc", mcamrc_dir);
sprintf(pathfilename_bak, "%s/.mcamrc.bak", mcamrc_dir);
unlink(pathfilename_bak);
rename(pathfilename, pathfilename_bak);
rename(pathfilename_temp, pathfilename);

return 1;
}/* end function save settings */

